@@ -12,12 +12,13 @@
 ##  Constraint
 ##  ==========
 ##' @title constraint
-##' @description \pkg{ROI} distinguishes between 4 different 
+##' @description \pkg{ROI} distinguishes between 5 different 
 ##'   types of constraint: 
 ##'   \itemize{
 ##'     \item No Constraint \code{\link{NO_constraint}} (inherits from \code{"constraint"})
 ##'     \item Linear Constraint \code{\link{L_constraint}} (inherits from \code{"constraint"})
 ##'     \item Quadratic Constraint \code{\link{Q_constraint}} (inherits from \code{"constraint"})
+##'     \item Conic Constraint \code{\link{C_constraint}} (inherits from \code{"constraint"})
 ##'     \item Function Constraint \code{\link{F_constraint}} (inherits from \code{"constraint"})
 ##'   }
 ##' @param x an object to be coerced or tested.
@@ -33,23 +34,41 @@ NULL
 ## 'constraints' helper functions
 ################################################################################
 
-available_constraint_classes <- function()
-    c(L = "L_constraint", Q = "Q_constraint", F = "F_constraint", X = "NO_constraint")
+available_constraint_classes <- function() {
+    c(L = "L_constraint", C = "C_constraint", Q = "Q_constraint", 
+      F = "F_constraint", X = "NO_constraint")
+}
+
+available_cone_types <- function()
+    c("zero", "nonneg", "soc", "psd", "expp", "expd", "powp", "powd")
+
+valid_cone <- function(x) x %in% available_cone_types()
 
 ################################################################################
 ## 'constraints' extractor functions
 ################################################################################
 
-##' Extract constraints from its argument (typically ROI objects) and
-##' return them.
-##'
-##' Currently, there is no default method. See \code{\link{constraints.OP}}
-##' for extracting constraints from ROI objects of class \code{"OP"}.
-##' @title Extract constraints
+##  Extract constraints from its argument (typically ROI objects) and
+##  return them.
+## 
+##  Currently, there is no default method. See \code{\link{constraints.OP}}
+##  for extracting constraints from ROI objects of class \code{"OP"}.
+##' @title Constraints - Accessor and Mutator Functions
+##' @description The \link{constraints} of a given optimization problem (\link{OP}) 
+##'     can be accessed or mutated via the method \code{'constraints'}.
 ##' @param x an object used to select the method.
 ##' @param value an R object.
 ##' @return the extracted constraints object.
+##' @name constraints (Set/Get)
+##' @rdname constraints
 ##' @author Stefan Theussl
+##' @examples
+##' ## minimize: x + 2 y
+##' ## subject to: x + y >= 1
+##' ## x, y >= 0
+##' x <- OP(1:2)
+##' constraints(x) <- L_constraint(c(1, 1), ">=", 1)
+##' constraints(x)
 ##' @export
 constraints <- function( x )
     UseMethod("constraints")
@@ -101,10 +120,40 @@ constraints.OP <- function( x ){
 ##' @noRd
 ##' @export
 'constraints<-.OP' <- function( x, value ) {
-    x$constraints <- as.constraint(value)
+    if ( is.null(value) ) {
+        x[["constraints"]] <- NO_constraint(length(objective(x)))
+    } else {
+        constr <- as.constraint(value)
+        .check_constraints(constr, x)
+        x$constraints <- constr
+    }
     x
 }
 
+.check_constraints <- function(constr, x) UseMethod(".check_constraints")
+
+.check_constraints.NULL <- function(constr, x) NULL
+
+.check_constraints.NO_constraint <- function(constr, x) NULL
+
+.check_constraints.L_constraint <- function(constr, x) {
+    if ( length(objective(x)) != ncol(constr) ) {
+        stop( "dimensions of 'objective' and 'constraints' not conformable." )
+    }
+    NULL
+}
+
+.check_constraints.C_constraint <- .check_constraints.L_constraint
+.check_constraints.Q_constraint <- .check_constraints.L_constraint
+
+.check_constraints.F_constraint <- function(constr, x) {
+    x0 <- rep.int(1, length(objective(x)))
+    F_len <- sum(unlist(lapply(constr$F, function(f) length(f(x0))), FALSE, FALSE)) 
+    if ( F_len != length(constr$rhs) ) {
+        stop( "dimensions of 'rhs' and 'constraints' not conformable." )
+    }
+    NULL
+}
 
 ## combining matrices (see 'rbind' in matrix.R, package relation)
 
@@ -113,9 +162,10 @@ constraints.OP <- function( x ){
 ##'
 ##' The output type is determined from the highest type of the
 ##' components in the hierarchy \cr \code{"L_constraint"} <
-##' \code{"Q_constraint"} < \code{"F_constraint"}.
+##' \code{"Q_constraint"} < \code{"F_constraint"} and \cr
+##' \code{"L_constraint"} < \code{"C_constraint"}.
 ##'
-##' @title Linear Constraints
+##' @title Combine Constraints
 ##' @param ... constraints objects to be concatenated.
 ##' @param use.names a logical if \code{FALSE} the names of the constraints
 ##'   are ignored when combining them, if \code{TRUE} the constraints are
@@ -141,6 +191,8 @@ rbind.constraint <- function(..., use.names=FALSE, recursive=FALSE) {
     if ( length( constraints ) == 1L ) return( constraints[[1L]] )
     if ( "F_constraint" %in% constraint_types ) {
         return( rbind_F_constraint(constraints) )
+    } else if ( "C_constraint" %in% constraint_types ) {
+        return( rbind_C_constraint(constraints) )
     } else if ( "Q_constraint" %in% constraint_types ) {
         return( rbind_Q_constraint(constraints, use.names=use.names) )
     } else if ( "L_constraint" %in% constraint_types ) {
@@ -177,7 +229,9 @@ c.constraint <- function( ..., recursive = FALSE )
 NO_constraint <- function( n_obj ) {
     n_obj <- as.integer( n_obj )
     stopifnot( length(n_obj) == 1L )
-    structure( list(),
+
+    L <- simple_triplet_zero_matrix(0, n_obj)
+    structure( list(L = L, dir = character(), rhs = numeric(), names = NULL),
                n_constraints = 0,
                n_obj = n_obj,
                class = c("NO_constraint", "constraint") )
@@ -224,16 +278,13 @@ is.NO_constraint <- function( x ) {
     inherits( x, "NO_constraint" )
 }
 
-rbind_NO_constraint <- function( ..., recursive = FALSE ){
-    constraints <- list(...)
+rbind_NO_constraint <- function(constraints) {
     nc <- unlist(lapply(constraints, is.NO_constraint))
-    dims <- unlist(lapply(constraints, function(x) dim(x)[1]))
+    dims <- unlist(lapply(constraints, function(x) dim(x)[2]))
     stopifnot( all(dims == dims[1]) )
-    ## FIXME: shouldn't we check dimensionaliy of constraints?
-    if( all(nc) )
-        constraints[[1]]
-    else
-        stop( "can only rbind objects of class 'NO_constraint'." )
+    if( !all(nc) )
+        stop( "constraints of type 'NO_constraint' can not be combined with other constraints." )
+    return( constraints[[1]] )
 }
 
 ##' @noRd
@@ -265,7 +316,7 @@ length.NO_constraint <- function( x )
 ##'
 ##' @title Linear Constraints
 ##' @param L a numeric vector of length \eqn{n} (a single constraint)
-##' or a matrix of dimension \eqn{n \times m}, where \eqn{n} is the
+##' or a matrix of dimension \eqn{m \times n}, where \eqn{n} is the
 ##' number of objective variables and \eqn{m} is the number of
 ##' constraints. Matrices can be of class
 ##' \code{"simple_triplet_matrix"} to allow a sparse representation of
@@ -447,8 +498,6 @@ rbind_L_constraint <- function( constraints, use.names = FALSE) {
     }
 }
 
-## FIXME: connection to rbind documentation
-
 ##' @noRd
 ##' @export
 c.L_constraint <- function( ..., recursive = FALSE )
@@ -468,21 +517,34 @@ length.L_constraint <- function( x )
 ##' @noRd
 ##' @export
 ##  c("n_constraints", "n_L_constraints", "n_Q_constraints", "n_F_constraints")
+##  The Problem is we overwrite length therefore str will fail since it iterates over the length.
+##  I seems easiest to just change the class information, so it get's printed correctly and
+##  length is not dispached to it's ROI implementations.
 str.constraint <- function(object, ...) {
-    i <- grep("n_.*constraints", names(attributes(qc)))
-    attributes(qc)[[i]] <- length(unclass(object))
-    str_default(object)
+    str(unclass(object))
+    cat(sprintf(' - attr(*, "class")='))
+    str(class(object))
 }
 
+##' @noRd
+##' @export
+str.cone <- function(object, ...) {
+    str(unclass(object), nest.lev = 1)
+    cat(sprintf(' - attr(*, "class")='))
+    str(class(object))
+}
 
 ##  the linear term of the left hand side
 ##  ----------------------------------------------------------
 ##  as.L_term
 ##  =========
-##' @title Coerce an object to type \code{"L_term"}.
+##  @title Coerce an object to type \code{"L_term"}.
+##' @title Canonicalize the Linear Term
 ##' @description
-##'   The \code{"L_term"} object represents the linear term of the \code{"L_constraint"}.
-##'   Objects from the following classes can be coerced to \code{"L_term"}:
+##    The \code{"L_term"} object represents the linear term of the \code{"L_constraint"}.
+##    Objects from the following classes can be coerced to \code{"L_term"}:
+##'   Canonicalize the linear term of a linear constraint.
+##'   Objects from the following classes can be canonicalized:
 ##'   \code{"NULL"}, \code{"numeric"}, \code{"matrix"}, \code{"simple_triplet_matrix"}
 ##'   and \code{"list"}.
 ##' @details
@@ -492,7 +554,7 @@ str.constraint <- function(object, ...) {
 ##'   with the specified dimension.
 ##' @param x an R object.
 ##' @param ... further arguments passed to or from other methods.
-##' @return an object of class \code{"simple_triplet_zero_matrix"}
+##' @return an object of class \code{"simple_triplet_matrix"}
 ##' @export
 as.L_term <- function( x, ... )
     UseMethod("as.L_term")
@@ -540,7 +602,7 @@ as.L_term.NULL <- function( x, ... ) {
 ##' @param Q a list of (sparse) matrices representing the quadratic
 ##' part of each constraint.
 ##' @param L a numeric vector of length \eqn{n} (a single constraint)
-##' or a matrix of dimension \eqn{n \times m}, where \eqn{n} is the
+##' or a matrix of dimension \eqn{m \times n}, where \eqn{n} is the
 ##' number of objective variables and \eqn{m} is the number of
 ##' constraints. Matrices can be of class
 ##' \code{"simple_triplet_matrix"} to allow a sparse representation of
@@ -687,10 +749,13 @@ length.Q_constraint <- function(x)
 ##  ----------------------------------------------------------
 ##  as.Q_term
 ##  =========
-##' @title Coerce an object to type \code{"Q_term"}.
+##  @title Coerce an object to type \code{"Q_term"}.
+##' @title Canonicalize the Quadraric Term
 ##' @description
-##'   The \code{"Q_term"} object represents the quadratic term of the \code{"Q_constraint"}.
-##'   Objects from the following classes can be coerced to \code{"Q_term"}:
+##    The \code{"Q_term"} object represents the quadratic term of the \code{"Q_constraint"}.
+##    Objects from the following classes can be coerced to \code{"Q_term"}:
+##'   Canonicalize the quadraric term of a quadratic constraint.
+##'   Objects from the following classes can be canonicalized:
 ##'   \code{"NULL"}, \code{"numeric"}, \code{"matrix"}, \code{"simple_triplet_matrix"}
 ##'   and \code{"list"}.
 ##' @details
@@ -700,7 +765,7 @@ length.Q_constraint <- function(x)
 ##'   with the specified dimension.
 ##' @param x an R object.
 ##' @param ... further arguments
-##' @return an object of class \code{"simple_triplet_zero_matrix"}
+##' @return an object of class \code{"simple_triplet_matrix"}
 ##' @export
 as.Q_term <- function(x, ...)
     UseMethod( "as.Q_term" )
@@ -744,15 +809,6 @@ as.Q_term.NULL <- function( x, ... ) {
     return( NULL )
 }
 
-## combine, print, and summary methods
-
-##summary.Q_constraint <- function(x){
-##
-##}
-
-
-## FIXME: Function constraints still incomplete and untested
-
 ################################################################################
 ## Function constraints (class 'F_constraint')
 ## list of constraints of the form f(x) ~ b
@@ -780,7 +836,11 @@ as.Q_term.NULL <- function( x, ... ) {
 ##' from \code{"constraint"}.
 ##' @author Stefan Theussl
 ##' @export
-F_constraint <- function(F, dir, rhs, J=NULL, names=NULL){
+## NOTE: checking the length of F_Constraints is a little more challenging since
+##       we allow a single F_Constraint to have a length greater than 1. Therefore
+##       to obtain the length we have to evaluate the constraints and get the length
+##       on the unlisted results.
+F_constraint <- function(F, dir, rhs, J=NULL, names=NULL) {
     stopifnot( row_sense_is_feasible(dir) )
     stopifnot( (length(F) == length(J)) | is.null(J) )
     F     <- as.F_term( F )
@@ -789,8 +849,7 @@ F_constraint <- function(F, dir, rhs, J=NULL, names=NULL){
     n_F   <- length( F )
     n_dir <- length( dir )
     n_F_constraints <- length( rhs )
-    ## length of F, dir and rhs need to be equal
-    stopifnot( all(c(n_F, n_dir) == n_F_constraints) )
+    stopifnot( n_dir == n_F_constraints )
     structure( list(F   = F,
                     dir = dir,
                     rhs = rhs,
@@ -850,17 +909,20 @@ is.F_constraint <- function( x ) {
     inherits( x, "F_constraint" )
 }
 
-rbind_F_constraint <- function( constraints ) {
+rbind_F_constraint <- function( constr ) {
     ## check if all inherit from constraint
-    constraints <- lapply(constraints, as.F_constraint)
-    fun <- unlist(lapply(constraints, "[[", "F"), use.names=FALSE)
-    dir <- unlist(lapply(constraints, "[[", "dir"), use.names=FALSE)
-    rhs <- unlist(lapply(constraints, "[[", "rhs"), use.names=FALSE)
-    jac <- unlist(lapply(constraints, "[[", "J"), use.names=FALSE)
-    names <- 
-    ## NOTE: should we check the dims? I currently don't since it should be correct
-    ##       as a result of the checks before
-    F_constraint(F=fun, dir=dir, rhs=rhs, J=jac, names=names)
+    constr <- lapply(constr, as.F_constraint)
+    cfun <- unlist(lapply(constr, "[[", "F"), use.names=FALSE)
+    cdir <- unlist(lapply(constr, "[[", "dir"), use.names=FALSE)
+    crhs <- unlist(lapply(constr, "[[", "rhs"), use.names=FALSE)
+    ## NOTE: We can just unlist it and if one constraint is missing 
+    ##       we just set them all NULL.
+    cjac <- unlist(lapply(constr, "[[", "J"), recursive=FALSE, use.names=FALSE)
+    if ( length(cjac) != length(cfun) ) cjac <- NULL
+    cnames <- lapply(constr, "[[", "names")
+    b <- !sapply(cnames, is.null)
+    cnames <- if (any(b)) cnames[b][[1]] else NULL
+    F_constraint(F=cfun, dir=cdir, rhs=crhs, J=cjac, names=cnames)
 }
 
 ##' @noRd
@@ -903,6 +965,10 @@ as.constraint.NULL <- identity
 ##' @export
 as.constraint.L_constraint <- identity
 
+##' @noRd
+##' @export
+as.constraint.C_constraint <- identity
+
 ##' @noRd
 ##' @export
 as.constraint.Q_constraint <- identity
@@ -922,30 +988,52 @@ is.constraint <- function(x) inherits(x, "constraint")
 
 ##' @noRd
 ##' @export
-print.constraint <- function( x, ... ){
+print.constraint <- function( x, ... ) {
     len <- length(x)
-    if ( is.NO_constraint(x) )
+    constr <- c("constraint", "constraints")
+    if ( is.NO_constraint(x) ) {
         writeLines( "An object of type 'NO_constraint'." )
-    if ( is.L_constraint(x) )
-        writeLines( sprintf("An object containing %d linear constraints.", len) )
-    else
-        if( is.Q_constraint(x) )
-            writeLines( c(sprintf("An object containing %d constraints.", len),
-                          "Some constraints are of type quadratic.") )
-        else
-            writeLines( c(sprintf("An object containing %d constraints.", len),
-                          "Some constraints are of type nonlinear.") )
+    } else if ( is.L_constraint(x) ) {
+        i <- 1L + as.integer(len != 1L)
+        txt <- sprintf("An object containing %d linear %s.", 
+                       len, constr[i])
+        writeLines( txt )
+    } else {
+        if( is.Q_constraint(x) ) {
+            b <- sapply(x$Q, is_zero_matrix)
+            n_L_constraints <- sum(b)
+            n_Q_constraints <- length(b) - n_L_constraints
+            fmt <- paste("An object containing %d linear %s",
+                         "                     %d quadratic %s.", sep = "\n")
+            i <- 1L + as.integer(n_L_constraints != 1L)
+            j <- 1L + as.integer(n_Q_constraints != 1L)
+            txt <- sprintf(fmt, n_L_constraints, constr[i],
+                           n_Q_constraints, constr[j])
+            writeLines( txt )
+        } else if ( is.C_constraint(x) ) {
+            i <- 1L + as.integer(len != 1L)
+            txt <- sprintf("An object containing %d conic %s.", 
+                           len, constr[i])
+            writeLines( txt )
+        } else {
+            i <- 1L + as.integer(len != 1L)
+            txt <- sprintf("An object containing %d nonlinear %s.", 
+                           len, constr[i])
+            writeLines( txt )
+        }
+    }
     invisible(x)
 }
 
 ##' @rdname ROI_constraint
 ##' @export
-dim.constraint <- function( x ){
-    ## FIXME: we should actually save both dimensions in constraint object
+dim.constraint <- function( x ) {
     out <- if( inherits(x, "NO_constraint") )
         c( length(x), attributes(x)$n_obj )
     else if( inherits(x, "L_constraint") )
         c( length(x), ncol(x$L))
+    else if( inherits(x, "C_constraint") )
+        c( length(x), ncol(x$L))
     else if( inherits(x, "Q_constraint") )
         c( length(x), ncol(x$L) )
     else if( inherits(x, "F_constraint") ){
@@ -954,33 +1042,12 @@ dim.constraint <- function( x ){
     out
 }
 
-## ---------------------------
-## terms
-## ---------------------------
-##' @rdname L_constraint
-##' @export
-terms.L_constraint <- function( x, ... ) {
-    list( L = x$L, dir=x$dir, rhs=x$rhs, names=x$names )
-}
-
-##' @rdname Q_constraint
-##' @export
-terms.Q_constraint <- function( x, ... ) {
-    list( Q = x$Q, L = x$L, dir=x$dir, rhs=x$rhs, names=x$names )
-}
-    
-##' @rdname F_constraint
-##' @export
-terms.F_constraint <- function( x, ... ) {
-    list( F = x$F, G = x$J, dir=x$dir, rhs=x$rhs, names=x$names )
-}
-
-
 ## ---------------------------
 ## as.function.constraint
 ## ---------------------------
 ##' @noRd
 ##' @export
+## FIXME as function C_constraint
 as.function.constraint <- function(x, ...) {
     if ( inherits(x, "L_constraint") ) return(as_function_L_constraint(x, ...))
     if ( inherits(x, "Q_constraint") ) return(as_function_Q_constraint(x, ...))
@@ -991,7 +1058,7 @@ as.function.constraint <- function(x, ...) {
 as_function_L_constraint <- function( x, ... ) {
     fun <- function(L_row_i) {        
         f <- function(x) c(tcrossprod(L_row_i, t(x)))
-        class(f) <- c(class(f), class(x))
+        ##class(f) <- c(class(f), class(x))
         return(f)
     }
     ## NOTE: rowapply_simple_triplet_matrix doesn't save L_row_i
@@ -1007,7 +1074,7 @@ as_function_Q_constraint <- function(x, ...) {
         f <- function (x) {    
             c(tcrossprod_simple_triplet_matrix(L, t(x)) + 0.5 * .xtQx(Q, x))
         }
-        class(f) <- c(class(f), class(x))
+        ## class(f) <- c(class(f), class(x))
         return(f)
     }
     out <- lapply(seq_len(NROW(x)), fun)
@@ -1018,7 +1085,6 @@ as_function_Q_constraint <- function(x, ...) {
 ## ---------------------------
 ## as.F_constraint
 ## ---------------------------
-##  FIXME: there are still F_constraint methods to implement
 ##' @rdname F_constraint
 ##' @export
 as.F_constraint <- function(x, ...) UseMethod( "as.F_constraint" )
@@ -1027,6 +1093,12 @@ as.F_constraint <- function(x, ...) UseMethod( "as.F_constraint" )
 ##' @export
 as.F_constraint.NULL <- function(x, ...) x
 
+##' @rdname F_constraint
+##' @export
+as.F_constraint.NO_constraint <- function(x, ...) x
+
+## <<< TODO: The typecasting from C_constraint to F_constraint is more
+##           challenging and in some chases doesn't make much sense. >>>
 ##' @rdname F_constraint
 ##' @export
 as.F_constraint.constraint <- function(x, ...) {
@@ -1036,3 +1108,354 @@ as.F_constraint.constraint <- function(x, ...) {
     stop("'x' must be of type L_constraint, Q_constraint or F_constraint, was ", shQuote(typeof(x)))
 }
 
+
+## ---------------------------
+## Cone Constraint
+## ---------------------------
+## params is a list of lists of parameters
+simple_cone <- function(n, type, params = NULL) {
+    id <- id_generator$get_ids(1L)
+    if ( !is.null(params) ) {
+        names(params) <- id
+    }
+    structure(list(cone = rep.int(type, n), 
+                   id = rep.int(id, n), 
+                   params = params), 
+              class = "cone")
+}
+
+##' @title Cone Constructors
+##' @description Constructor functions for the different cone
+##'     types. Currently \pkg{ROI} supports eight different types
+##'     of cones.
+##'     \itemize{
+##'     \item \code{Zero cone} \deqn{ \mathcal{K}_{\mathrm{zero}} = \{0\} }
+##'     \item \code{Nonnegative (linear) cone} \deqn{ \mathcal{K}_{\mathrm{lin}} = \{x|x \geq 0 \} }
+##'     \item \code{Second-order cone} \deqn{ \mathcal{K}_{\mathrm{soc}} = \left\{(t, x) \ | \ ||x||_2 \leq t, x \in R^n, t \in R \right\} }
+##'     \item \code{Positive semidefinite cone} \deqn{ \mathcal{K}_{\mathrm{psd}} = \left\{ X \ | \ min(eig(X)) \geq 0, \ X = X^T, \ X \in R^{n \times n} \right\} }
+##'     \item \code{Exponential cone} \deqn{ \mathcal{K}_{\mathrm{expp}} = \left\{(x,y,z) \ | \ y e^{\frac{x}{y}} \leq z, \ y > 0 \right\} }
+##'     \item \code{Dual exponential cone} \deqn{ \mathcal{K}_{\mathrm{expd}} = \left\{(u,v,w) \ | \ -u e^{\frac{v}{u}} \leq e w, u < 0 \right\} }
+##'     \item \code{Power cone} \deqn{ \mathcal{K}_{\mathrm{powp}} = \left\{(x,y,z) \ | \ x^\alpha * y^{(1-\alpha)} \geq |z|, \ x \geq 0, \ y \geq 0 \right\} }
+##'     \item \code{Dual power cone} \deqn{ \mathcal{K}_{\mathrm{powd}} = \left\{ (u,v,w) \ | \ \left(\frac{u}{\alpha}\right)^\alpha * \left(\frac{v}{(1-\alpha)}\right)^{(1-\alpha)} \geq |w|, \ u \geq 0, \ v \geq 0 \right\} }
+##'     }
+##' 
+##' @param size a integer giving the size of the cone, 
+##'        if the dimension of the cones is fixed 
+##'        (i.e. \code{zero}, \code{lin}, \code{expp}, \code{expd})
+##'        the number of cones is sufficient to define the dimension
+##'        of the product cone.
+##' @param sizes a integer giving the sizes of the cones,
+##'        if the dimension of the cones is not fixed 
+##'        (i.e. \code{soc}, \code{psd}) we
+##'        have to define the sizes of each single cone.
+##' @param alpha a numeric vector giving the \code{alphas}
+##'        for the (dual) power cone.
+##' @rdname cone
+##' @examples 
+##' K_zero(3) ## 3 equality constraints
+##' K_lin(3)  ## 3 constraints where the slack variable s lies in the linear cone
+##' @export
+K_zero <- function(size) {
+    simple_cone(size, 1L)
+}
+
+##' @rdname cone
+##' @export
+K_lin <- function(size) {
+    simple_cone(size, 2L)
+}
+
+##' @rdname cone
+##' @export
+K_soc <- function(sizes) {
+    do.call(c, lapply(sizes, simple_cone, type = 3L))
+}
+
+##' @rdname cone
+##' @export
+K_psd <- function(sizes) {
+    do.call(c, lapply(sizes, simple_cone, type = 4L))
+}
+
+##' @rdname cone
+##' @export
+K_expp <- function(size) {
+    do.call(c, lapply(rep.int(3L, size), simple_cone, type = 5L))
+}
+
+##' @rdname cone
+##' @export
+K_expd <- function(size) {
+    do.call(c, lapply(rep.int(3L, size), simple_cone, type = 6L))
+}
+
+##' @rdname cone
+##' @export
+K_powp <- function(alpha) {
+    fun <- function(x) simple_cone(3L, 7L, params = list(c(a = x)))
+    do.call(c, lapply(alpha, fun))
+}
+
+##' @rdname cone
+##' @export
+K_powd <- function(alpha) {
+    fun <- function(x) simple_cone(3L, 8L, params = list(c(a = x)))
+    do.call(c, lapply(alpha, fun))
+}
+
+##' @noRd
+##' @export
+c.cone <- function(...) {
+    x <- list(...)
+    cone <- list()
+    cone$cone <- do.call(c, lapply(x, "[[", "cone"))
+    cone$id <- do.call(c, lapply(x, "[[", "id"))
+    cone$params <- do.call(c, lapply(x, "[[", "params"))
+    class(cone) <- "cone"
+    cone
+}
+
+##' @noRd
+##' @export
+`[.cone` <- function(x, i) {
+    x$cone <- x$cone[i]
+    x$id <- x$id[i]
+    if ( !is.null(x$params) ) {
+        j <- which(x$id %in% as.integer(names(x$params)))
+        x$params <- x$params[j]
+    }
+    x
+}
+
+##' @noRd
+##' @export
+length.cone <- function(x) {
+    length(x$cone)
+}
+
+calc_dims <- function(x, type) {
+    i <- which(x$cone == type)
+    ids <- x[,'id']
+    if ( length(i) ) {
+        if (type %in% 1:2) {
+            dims <- length(ids[[i]])
+        } else if (type %in% 3:4) {
+            dims <- table(ids[[i]])
+        } else if (type %in% 5:8) {
+            dims <- length(unique(ids[[i]]))
+        }
+    }
+    dims
+}
+
+################################################################################
+##  C_constraint
+##  ============
+##' @title Conic Constraints
+##' @description
+##' Conic constraints are often written in the form \deqn{Lx + s = rhs}
+##' where \eqn{L} is a \eqn{m \times n} (sparse) matrix and 
+##' \eqn{s \in \mathcal{K}} are the slack variables restricted to
+##' some cone \eqn{\mathcal{K}} which is typically the product of simpler cones 
+##' \eqn{\mathcal{K} = \prod \mathcal{K}_i}. The right hand side \eqn{rhs} 
+##' is a vector of length \eqn{m}.
+##'
+
+##' @param L a numeric vector of length \eqn{n} (a single constraint)
+##' or a matrix of dimension \eqn{m \times n}, where \eqn{n} is the
+##' number of objective variables and \eqn{m} is the number of
+##' constraints. Matrices can be of class
+##' \code{"simple_triplet_matrix"} to allow a sparse representation of
+##' constraints.
+##' @param cones an object of class \code{"cone"} created by the combination,
+##'        of \code{\link{K_zero}}, \code{\link{K_lin}}, \code{\link{K_soc}}, 
+##'        \code{\link{K_psd}}, \code{\link{K_expp}}, \code{\link{K_expd}}, 
+##'        \code{\link{K_powp}} or \code{\link{K_powd}}.
+##' @param rhs a numeric vector giving the right hand side of the constraints.
+##' @param names an optional character vector giving the names of \eqn{x} 
+##'        (column names of \eqn{L}).
+##' @param x an R object.
+##' @param ... further arguments passed to or from other methods
+##' (currently ignored).
+##' @return an object of class \code{"C_constraint"} which inherits
+##'         from \code{"constraint"}.
+##' @examples
+##' ## minimize:  x1 + x2 + x3
+##' ## subject to: 
+##' ##   x1 == sqrt(2)
+##' ##   ||(x2, x3)|| <= x1
+##' x <- OP(objective = c(1, 1, 1), 
+##'         constraints = C_constraint(L = rbind(rbind(c(1, 0, 0)), 
+##'                                              diag(x=-1, 3)), 
+##'                                    cones = c(K_zero(1), K_soc(3)), 
+##'                                    rhs = c(sqrt(2), rep(0, 3))), 
+##'         types = rep("C", 3),
+##'         bounds =  V_bound(li = 1:3, lb = rep(-Inf, 3)), maximum = FALSE)
+##' @export
+C_constraint <- function(L, cones, rhs, names = NULL) {
+    L     <- as.L_term(L)
+    rhs   <- as.rhs( rhs )
+    dim_L <- dim( L )
+    n_L_constraints <- length( rhs )
+    stopifnot(length(cones) == length(rhs), length(rhs) == nrow(L))
+    if ( (!is.null(names)) & (length(names) != ncol(L)) ) {
+        stop("number of columns of 'L' and length 'names' must be equal.")
+    }
+    structure( list(L     = L,
+                    cones = cones,
+                    rhs   = rhs,
+                    names = names),
+              n_L_constraints = n_L_constraints,
+              class = c("C_constraint", "constraint") )
+}
+
+
+##' @rdname C_constraint
+##' @export
+as.C_constraint <- function(x, ...) UseMethod("as.C_constraint")
+
+##' @noRd
+##' @export
+as.C_constraint.C_constraint <- function(x, ...) identity(x)
+
+##' @noRd
+##' @export
+as.C_constraint.NO_constraint <- function(x, ...) identity(x)
+
+##' @noRd
+##' @export
+as.C_constraint.NULL <- function(x, ...) identity(x)
+
+##' @noRd
+##' @export
+as.C_constraint.L_constraint <- function(x, ...) {
+    if ( !nrow(x) ) {
+        x$dir <- simple_cone(0, 2)
+        names(x)[names(x) == "dir"] <- "cones"
+        return(x)
+    }
+
+    cdir <- aggregate(id ~ dir, 
+                      data = list(id = seq_along(x$dir), dir = x$dir), 
+                      FUN = c, simplify = FALSE)
+
+    names(x)[names(x) == "dir"] <- "cones"
+    
+    ## <=
+    ## canonical case: I don't need to do anything!
+
+    ## >=
+    i <- which(cdir$dir == ">=")
+    if ( length(i) ) {
+        i <- cdir[[i, "id"]]
+        x$rhs[i] <- -x$rhs[i]
+        j <- which(x$L$i %in% i)
+        x$L$v[j] <- -x$L$v[j]
+    }
+
+    x$cones <- K_lin(length(x$cones))
+    
+    ## == (zero cone)
+    i <- which(cdir$dir == "==")
+    if ( length(i) ) {
+        i <- cdir[[i, "id"]]
+        zero <- K_zero(length(i))
+        x$cones$cone[i] <- zero$cone
+        x$cones$id[i] <- zero$id
+    }
+       
+    class(x) <- c("C_constraint", "constraint")
+    x
+}
+
+##  Tests if an object is interpretable as being of class \code{"C_constraint"}.
+## 
+##  @title Conic Constraints
+##  @param x object to be tested.
+##  @return returns \code{TRUE} if its argument is of class
+##  \code{"C_constraint"} and \code{FALSE} otherwise.
+##' @rdname C_constraint
+##' @export
+is.C_constraint <- function( x ) {
+    inherits(x, "C_constraint")
+}
+
+##  Get the number of constraints from a corresponding ROI object.
+## 
+##  @title Linear Constraints
+##  @param x constraints object.
+##  @return an integer.
+##' @rdname C_constraint
+##' @export
+length.C_constraint <- function( x )
+    attr(x, "n_L_constraints")
+
+##' @rdname C_constraint
+##' @param object an R object.
+##' @export
+variable.names.C_constraint <- function(object, ...) {
+    object$names
+}
+
+## combine 2 C_constraints (ignore names)
+c2_C_constraints <- function(x, y) {
+    C_constraint( L=rbind(x$L, y$L), cones=c(x$cones, y$cones),  
+                  rhs=c(x$rhs, y$rhs), names=x$names )
+}
+
+## combine 2 C_constraints (use names, using the names allows to 'rbind' 
+## 2 C_constraints where the number of columns to not match by matching the names)
+c2_C_constraints_named <- function(x, y) {
+    C_constraint( L=rbind_stm_by_names(x$L, y$L, variable.names(x), variable.names(y)),
+                  cones=c(x$cones, y$cones), rhs=c(x$rhs, y$rhs), 
+                  names=variable.names(x) )
+}
+
+rbind_C_constraint <- function( constraints, use.names = FALSE) {
+    constraints <- lapply(constraints, as.C_constraint)
+    if ( length(constraints) == 1L ) return( constraints[[1L]] )
+    if ( use.names ) {
+        var.names <- lapply(constraints, function(x) variable.names(x))
+        if ( any(sapply(var.names, is.null)) ) {
+            stop("all constraints need to be named if 'use.names' is TRUE!")
+        }
+        is_equal <- function(x, y) isTRUE(all.equal(x, y))
+        if ( all(mapply(is_equal, var.names[-1L], var.names[-length(var.names)])) ) {
+            return( Reduce(c2_C_constraints, constraints) )
+        } else {
+            var.names <- unique(unlist(var.names))
+            lc <- C_constraint(simple_triplet_zero_matrix(0L, length(var.names)),
+                               NULL, NULL, names=var.names)
+            return( Reduce(c2_C_constraints_named, c(list(lc), constraints)) )
+        }
+    } else {
+        return( Reduce(c2_C_constraints, constraints) )
+    }
+}
+
+## ---------------------------
+## terms
+## ---------------------------
+##' @rdname L_constraint
+##' @export
+terms.L_constraint <- function( x, ... ) {
+    list( L = x$L, dir=x$dir, rhs=x$rhs, names=x$names )
+}
+
+##' @rdname C_constraint
+##' @export
+terms.C_constraint <- function( x, ... ) {
+    list( L = x$L, cones=x$cones, rhs=x$rhs, names=x$names )
+}
+
+##' @rdname Q_constraint
+##' @export
+terms.Q_constraint <- function( x, ... ) {
+    list( Q = x$Q, L = x$L, dir=x$dir, rhs=x$rhs, names=x$names )
+}
+    
+##' @rdname F_constraint
+##' @export
+terms.F_constraint <- function( x, ... ) {
+    list( F = x$F, J = x$J, dir=x$dir, rhs=x$rhs, names=x$names )
+}
